module Blufin

    class Arrays

        # Checks if an array has duplicates.
        # @return boolean
        def self.array_has_duplicate(array, ignore_case = false)
            raise RuntimeError, "Expected Array, but got #{array.class}" unless array.is_a?(Array)
            array = array.dup
            if ignore_case
                array.map! { |name| name.downcase }
            end
            !(array.uniq.length == array.length)
        end

        # Converts an Array of lines to a string (for easier gsub replacement).
        # @return String
        def self.convert_line_array_to_string(array_of_lines)
            raise RuntimeError, "Expected Array of lines, instead got: #{array_of_lines.class}" unless array_of_lines.is_a?(Array)
            string = ''
            array_of_lines.each_with_index do |line, idx|
                newline_or_not = (idx == (array_of_lines.length - 1)) ? '' : "\n"
                string         += "#{line}#{newline_or_not}"
            end
            string
        end

        # Converts a string to an Array of lines to a string.
        # @return String
        def self.convert_string_to_line_array(string)
            raise RuntimeError, "Expected String, instead got: #{string.class}" unless string.is_a?(String)
            array_of_lines = []
            string.split("\n").each { |line| array_of_lines << line.gsub("\n", '') }
            array_of_lines
        end

    end

end