module Blufin

    class Constants

        # Returns a Hash of gems. :repo is only filled in inside 'my' gem because it's the only gem where it's defined in the config file (.my.yml).
        # Outside this gem, :repo will simply be nil.
        # @return Hash
        def self.gems

            {
                'blufin-lib' => {
                    :name          => 'blufin-lib',
                    :aliases       => %w(blufin-lib bl),
                    :repo          => gem_get_repo('BlufinLib'),
                    :version_file  => 'lib/version.rb',
                    :version_const => 'BLUFIN_LIB_VERSION',
                    :downstream    => %w(convoy awx blufin eworld my), # Needs to be in this specific order, with convoy first.
                    :upstream      => %w()
                },
                'convoy'     => {
                    :name          => 'convoy',
                    :aliases       => %w(convoy),
                    :repo          => gem_get_repo('Convoy'),
                    :version_file  => 'version.rb',
                    :version_const => 'CONVOY_VERSION',
                    :downstream    => %w(awx blufin my eworld),
                    :upstream      => %w(blufin-lib)
                },
                'columnist'  => {
                    :name          => 'columnist',
                    :aliases       => %w(columnist),
                    :repo          => gem_get_repo('Columnist'),
                    :version_file  => 'version.rb',
                    :version_const => 'COLUMNIST_VERSION',
                    :downstream    => %w(awx blufin my eworld),
                    :upstream      => %w()
                },
                'awx'        => {
                    :name          => 'awx',
                    :aliases       => %w(awx a),
                    :repo          => gem_get_repo('AWX'),
                    :version_file  => 'lib/version.rb',
                    :version_const => 'AWX_VERSION',
                    :downstream    => %w(),
                    :upstream      => %w(blufin-lib columnist convoy)
                },
                'blufin'     => {
                    :name          => 'blufin',
                    :aliases       => %w(blufin b bf),
                    :repo          => gem_get_repo('Blufin'),
                    :version_file  => 'lib/version.rb',
                    :version_const => 'BLUFIN_VERSION',
                    :downstream    => %w(),
                    :upstream      => %w(blufin-lib columnist convoy)
                },
                'my'         => {
                    :name          => 'my',
                    :aliases       => %w(my m),
                    :repo          => gem_get_repo('My'),
                    :version_file  => 'lib/version.rb',
                    :version_const => 'MY_VERSION',
                    :downstream    => %w(),
                    :upstream      => %w(blufin-lib columnist convoy)
                },
                'eworld'     => {
                    :name          => 'eworld',
                    :aliases       => %w(ew e),
                    :repo          => gem_get_repo('eWorld'),
                    :version_file  => 'lib/version.rb',
                    :version_const => 'EWORLD_VERSION',
                    :downstream    => %w(),
                    :upstream      => %w(blufin-lib columnist convoy)
                }
            }

        end

        private

        # Attempts to get repo. Returns nil if not exists.
        # @return string
        def self.gem_get_repo(key)
            begin
                Blufin::Config::get['Repositories'][key]
            rescue
                nil
            end
        end

    end

end